module Main ( main ) where

import Data.Char (isSpace)
import Control.Applicative ( (<$>) )

import Dict

data Action = CreateEmpty | Add Word Def | Delete Word | Search Word | Size
            | Show | Save FilePath | Load FilePath | Quit


trim :: String -> String
trim = f . f
   where f = reverse . dropWhile isSpace

clearScreen :: IO ()
clearScreen = sequence_ (replicate 80 (putChar '\n'))

printMenu :: IO ()
printMenu = 
    let menu = ["***************************************",
                "* v: Vaciar diccionario               *",
                "* a: Agregar definicion               *",
                "* d: Borrar definicion                *",
                "* b: Buscar definicion                *",
                "* t: Ver el taman~o del diccionario   *",
                "* s: Mostrar diccionario              *",
                "* c: Cargar diccionario desde archivo *",
                "* g: Grabar diccionario a un archivo  *",
                "* q: Salir                            *",
                "***************************************"
               ]
    in 
        putStrLn $ unlines menu

getActionMenu :: IO Action
getActionMenu = 
    let

        getWord :: IO Word
        getWord = do
          putStrLn "Ingrese la palabra: "
          getLine

        getDef :: IO Def
        getDef = do
          putStrLn "Ingrese la definicion: "
          getLine

        getNameFile :: IO String
        getNameFile = do
          putStrLn "Ingrese el nombre de archivo: "
          str <- getLine
          putStrLn ""
          return (str++".dic")

        getAdd, getDelete, getLoad, getSave :: IO Action

        getAdd = do
          w <- trim <$> getWord
          d <- getDef
          return (Add w d)

        getDelete = do
          w <- trim <$> getWord
          return (Delete w)

        getSearch = do
          w <- trim <$> getWord
          return (Search w)

        getLoad = do
          f <- getNameFile
          return (Load f)

        getSave = do
          f <- getNameFile
          return (Save f)
    in do
        printMenu
        putStrLn "Ingrese una opcion y despues [Intro] "
        c <- getChar
        getChar
        clearScreen
        case c of
          'v' -> return CreateEmpty
          'a' -> getAdd
          'd' -> getDelete
          'b' -> getSearch
          't' -> return Size
          's' -> return Show
          'c' -> getLoad
          'g' -> getSave
          'q' -> return Quit
          _   -> getActionMenu

stepMain :: Dict -> IO (Action,Dict)
stepMain d = 
    let
        showSize :: Dict -> IO()
        showSize d = 
              putStrLn $ "Hay " ++ show (dict_length d)
                          ++ " palabras en el diccionario"

        showSearch :: Word -> Maybe Def -> IO()
        showSearch w Nothing = putStrLn "La palabra no existe\n"
        showSearch w (Just d) = 
              putStrLn $ w ++ " : " ++ d

        showDef :: (Word,Def) -> IO ()
        showDef (w,d) = 
              putStrLn $ w ++ " : " ++ d

        showDefs :: [(Word,Def)] -> IO()
        showDefs = mapM_ showDef
    in
      do a <- getActionMenu
         case a of
           CreateEmpty -> return (a, dict_empty)
           Add w df -> return (a, dict_add d w df)
           Delete w -> return (a, dict_del d w)
           Search w -> do showSearch w (dict_search d w) 
                          return (a, d)
           Size ->  do showSize d
                       return (a,d)
           Show ->  do showDefs (dict_toDefs d)
                       return (a,d)
           Load f -> do d' <- dictLoad f
                        showSize d
                        return (a,d')
           Save f -> do dictSave d f
                        return (a,d)
           Quit -> return (a,d)


dictLoad :: FilePath -> IO Dict
dictLoad f =
    let

        subsecs :: (a -> Bool) -> [a] -> [[a]]
        subsecs p = foldr f [[]]
            where
              f x (ps:pss) | p x       = (x:ps):pss
                           | not (p x) = []:ps:pss
        lineas :: String -> [String]
        lineas str = filter (/=[]) (subsecs (/='\n') str)

        lineToPair :: String -> (Word,Def)
        lineToPair str = case span (/=':') str of
                           ("",_) -> error "Palabra vacia"
                           (w,":") -> error ("Definicion vacia en palabra "
                                             ++ w)
                           (w,"") -> error ("Definicion vacia en palabra "
                                             ++ w)
                           (w, ':':d) -> (trim w, trim d)

        dictFromString :: String -> Dict
        dictFromString str = 
            foldr f dict_empty (map lineToPair (lineas str))
                where
                  f (k,d) di = dict_add di k d 
    in
      do
        str <- readFile f
        return (dictFromString str)

 
dictSave :: Dict -> FilePath -> IO ()
dictSave d f =
    let
        defsToString :: [(Word,Def)] -> String
        defsToString wfs = unlines (map f wfs)
            where
              f (w,d) = w ++ ":" ++ d
    in
      do
        writeFile f $ defsToString (dict_toDefs d)

cycleMain :: Dict -> IO ()
cycleMain d = do
  (a,d') <- stepMain d
  case a of
    Quit -> return ()
    _    -> cycleMain d'

main :: IO ()
main = 
    do 
      clearScreen
      cycleMain dict_empty
